﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'Loosely based on REFERENCE (2)
Sub Class_Globals

	Private ABM As ABMaterial
	
	Public Page As ABMPage
	
	'Browsers unique page id
	Private PageID As String = ""
	
	'Page name, must be same as this class name (case sensitive!)
	Public PageName As String = "Cam_4_ABM_Page"  '<---- IMPORTANT
	
	Private ws As WebSocket
	
	Private Cam_4_ABM_Instance As Cam_4_ABM_Object
	
End Sub

Public Sub Initialize
	
	'Initialize page
	Page.Initialize(PageName, "/ws/" & Main.Server.AppName & "/" & PageName, False, Main.Server.SessionMaxInactiveIntervalSeconds)
	Page.ShowLoader=True
	Page.PageHTMLName = "index.html"
	Page.PageTitle = "Cam_4_ABM"
	Page.PageDescription = "Cam_4_ABM"
	Page.PageKeywords = ""
	Page.PageSiteMapPriority = ""
	Page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
		
	'Green dot in top RH corner => connected, red dot => not connected
	Page.ShowConnectedIndicator = True

	'Create page grid - 1 row x 1 column
	Page.AddRowsM2(1, False, 0, 0, 0, 0, "").AddCells12MP(1, 0, 0, 0, 0, "")
	Page.BuildGrid 
	
	'Initialize instance of Cam_4_ABM_Object,
	'Passed_Instance_Name = name of this instance
	Cam_4_ABM_Instance.Initialize(Page, "Cam_4_ABM_Instance")

	'See Cam_4_ABM_Object.Initialize for defaults which
	'can be overidden here, for example:
'	Cam_4_ABM_Instance.BorderColor = ABM.COLOR_BLUE
'	Cam_4_ABM_Instance.BorderIntensity = ABM.INTENSITY_DARKEN2
'	Cam_4_ABM_Instance.BorderStyle = ABM.BORDER_SOLID
'	Cam_4_ABM_Instance.BorderWidth = "10px"
'	Cam_4_ABM_Instance.Borderradius = "10px"
'	Cam_4_ABM_Instance.PaddingBottom = "10px"
'	Cam_4_ABM_Instance.PaddingLeft = "10px"
'	Cam_4_ABM_Instance.PaddingRight = "10px"
'	Cam_4_ABM_Instance.PaddingTop = "10px"
'	Cam_4_ABM_Instance.ZDepth = ABM.ZDEPTH_REMOVE

	Page.Cell(1,1).AddComponent(Cam_4_ABM_Instance.ABMComp)
	
End Sub

Private Sub WebSocket_Connected(WebSocket1 As WebSocket)
	
	ws = WebSocket1	
	
	'This avoids error
	'    org.eclipse.jetty.websocket.api.exceptions.MessageTooLargeException: Text message too large: (actual) 66,704 > (configured max text message size) 65,536
	'when taking a picture, see:
	'    https://www.b4x.com/android/forum/threads/webapp-jetty-how-to-use-setmaxtextmessagesize.45644/post-983937
	'need to make wssize larger than max resolution of largest
	'screen expected to encounter - note screen, not camera
	Private wssize As Long = 4 * 1024 * 1024
	Private wrk_jo As JavaObject = ws
	wrk_jo = wrk_jo.GetFieldJO("session").RunMethod("getPolicy", Null)
	wrk_jo.RunMethod("setMaxTextMessageSize", Array(wssize))
	
	PageID = ABM.GetPageID(Page, PageName, ws)
	
	Main.Server.Connected(Me, ws, Page, PageID)
	
'	Log("Connected: " & PageID)
	
End Sub

Private Sub WebSocket_Disconnected
	
'	Log("Disconnected: " & PageID)
	
	Main.Server.Disconnected(ws)
	
End Sub

Sub Page_ParseEvent(Params As Map)
	
	Main.Server.Page_ParseEvent(Me, Page, PageID, Params)
	
End Sub

'MUST be here
Public Sub ConnectPage()
	
	'Refresh page
	Page.Refresh	
	
	'Tell browser finished loading
	Page.FinishedLoading
	
End Sub

Sub Cam_4_ABM_Instance_Ready(value As Map)
	Private wrk_out As String = value.GetDefault("value","")
	Page.ShowToast("toast", "toastgreen", wrk_out, 3000, False)
End Sub

Sub Cam_4_ABM_Instance_Error(value As Map)
	Private wrk_out As String = value.GetDefault("value","")
	Page.ShowToast("toast", "toastred", wrk_out, 3000, False)
End Sub

Sub Cam_4_ABM_Instance_ChromeBug(value As Map)
	
	'There appears to be a bug in Chrome mobile which is not in Safari
	'mobile or Windows 10 Chrome/Edge/Firefox - you can toggle from
	'[facingMode: 'user'] to [facingMode: 'environment'] but not back
	'again (or reverse if default is [facingMode: 'environment'])
	
	Private wrk_out As String = value.GetDefault("value", "")

	'Bug manifests as error message "Could not start video source"...
	If wrk_out = "Could not start video source" Then
		
'		LogError("<<<<<<<<<<<<<<<<<<<<<<<<<<<< CHROME BUG")
		
		'Reload page which will set [facingMode:] back to original
		Main.Server.NavigateToPage(ws, "" , "./")
		
	End If
	
End Sub

Sub Cam_4_ABM_Instance_PictureTaken(value As Map)
	
	Private wrk_out As String = value.GetDefault("value","")
	
	'Clean up image
	wrk_out = wrk_out.Replace("data:image/jpeg;base64,", "")

	'Images can not exist in a non-UI app so save string - to be converted
	'separately by a UI app 
	File.WriteString("D:\Cam_4_ABM_Photos\Base64Strings", PageID & ".txt", wrk_out)
	
	'As an example converter, see Cam_4_ABM_Converter B4J project
	
	'Following is for illustration only - it is all very primitive and
	'would need to be modified for a true production environment
	
	'Assumptions:
	'o Cam_4_ABM and Cam_4_ABM_converter B4J projects are installed in
	'  same main directory
	'o Cam_4_ABM_Converter.exe has been created via B4J IDE 
	'  Project/Build Standalone Package
	
	'Based on above assumptions, work out path to Cam_4_ABM_Converter.exe
	Private wrk_cmd As String
	wrk_cmd = File.DirApp
	wrk_cmd = wrk_cmd.Replace("Cam_4_ABM\", "Cam_4_ABM_Converter\")
	wrk_cmd = """" & wrk_cmd & "\temp\build\Cam_4_ABM_Converter.exe"""

	'Shell to Cam_4_ABM_Converter
	Private wrk_sh As Shell
	Private wrk_args As List
	wrk_args.Initialize
	wrk_args.Add("/c")
	wrk_args.Add(wrk_cmd)
	wrk_sh.Initialize("Event_shell", "cmd.exe", wrk_args)
							
	Try
								
		'Run shell with a 10 sec timeout
		wrk_sh.Run(10 * DateTime.TicksPerSecond)
		
		'Wait for shell to terminate...
		Wait For Event_shell_ProcessCompleted(Success As Boolean, ExitCode As Int, StdOut As String, StdErr As String)
		
'		Log("Cam_4_ABM_Converter.exe Success: " & Success)
'		Log("Cam_4_ABM_Converter.exe ExitCode: " & ExitCode)
'		Log("Cam_4_ABM_Converter.exe StdOut: " & StdOut)
'		Log("Cam_4_ABM_Converter.exe StdErr: " & StdErr)
		
	Catch
								
		LogError("Oops - Cam_4_ABM_Converter.exe bombed")
							
	End Try
							
End Sub
